==> de-BBBike-3.01.tgz: BBBike/BBBikeProfil.pm <==
# -*- perl -*-

#
# $Id: BBBikeProfil.pm,v 1.2 2000/09/19 23:24:16 eserte Exp $
# Author: Slaven Rezic
#
# Copyright (C) 1999 Slaven Rezic. All rights reserved.
# This package is free software; you can redistribute it and/or
# modify it under the same terms as Perl itself.
#
# Mail: eserte@cs.tu-berlin.de
# WWW:  http://user.cs.tu-berlin.de/~eserte/
#

package BBBikeProfil;
use BBBikeUtil;
use strict;
use vars qw();

sub new {
    bless {}, $_[0];
}

sub Show {
    my($self, $top, $context, %args) = @_;
    return if $self->{Destroyed};
    my $toplevel;
    if ($context->{ProfilToplevel} &&
	Tk::Exists($context->{ProfilToplevel})) {
	$toplevel = $context->{ProfilToplevel};
    } else {
	$toplevel = $top->Toplevel(-title => "Profil");
	$toplevel->OnDestroy(sub { $self->{Destroyed} = 1});
	$toplevel->transient($top) if $context->{Transient};
	$context->{ProfilToplevel} = $toplevel;

	my @hooks = qw/new_route del_route/;
	foreach my $hook (@hooks) {
	    Hooks::get_hooks($hook)->add
		    (sub { $self->Show($top, $context, %args) }, "profil");
	}
	$toplevel->OnDestroy
	    (sub {
		 foreach my $hook (@hooks) {
		     Hooks::get_hooks($hook)->del("profil");
		 }
	     }
	    );
    }
    my $c;
    if ($context->{ProfilCanvas} &&
	Tk::Exists($context->{ProfilCanvas})) {
	$c = $context->{ProfilCanvas};
    } else {
	my($w, $h) = (int($top->screenwidth/3*2),
		      int($top->screenheight/6));
	$c = $toplevel->Canvas(-height => $h, -width => $w)->pack;
	$context->{ProfilCanvas} = $c;
    }

    $self->Redraw($context, %args);
}

# XXX bind: bei Klick wird die entsprechende Stelle auf der Karte
# gezeigt. Vielleicht brauche ich hier auch die canvascoords oder
# die transpose-funktion
sub Redraw {
    my($self, $context, %args) = @_;
    my $t = $context->{ProfilToplevel};
    my $c = $context->{ProfilCanvas};
    $c->delete('all');
    my $hoehe   =    $context->{Hoehe};
    my(@coords) = @{ $context->{Coords} };
    my($w, $h) = ($c->cget(-width), $c->cget(-height)); 
    my(@dist) = (0);
    for(my $i=0; $i<$#coords; $i++) {
	my($x1,$y1, $x2,$y2) = (@{ $coords[$i] },
				@{ $coords[$i+1] });
	my $dist = CORE::sqrt(sqr($x2-$x1)+sqr($x2-$x1));
	push @dist, $dist[$#dist] + $dist;
    }

    my $max_h;
    for(my $i=0; $i<=$#coords; $i++) {
	my($x,$y) = @{ $coords[$i] };
	if (exists $hoehe->{"$x,$y"} &&
	    (!defined $max_h || $max_h < $hoehe->{"$x,$y"})) {
	    $max_h = $hoehe->{"$x,$y"};
	}
    }

    if (!defined $max_h) {
	print STDERR "Es gibt keine Hheninformationen fr diese Route\n";
	return;
    }

    my $x_sub = sub { int($_[0]/$dist[$#dist]*$w) };
    my $y_sub = sub { $h - int($_[0]/($max_h+10)*$h) };

    my($lastx, $lasty);
    for(my $i=0; $i<=$#dist; $i++) {
	my($d) = $dist[$i];
	my($x,$y) = @{ $coords[$i] };
	if (exists $hoehe->{"$x,$y"}) {
	    my($thisx, $thisy);
	    $thisx = $x_sub->($d);
	    $thisy = $y_sub->($hoehe->{"$x,$y"});
	    if (defined $lastx) {
		$c->createLine($lastx, $lasty, $thisx, $thisy);
	    }
	    ($lastx, $lasty) = ($thisx, $thisy);
	}
    }

    for(my $i=0; $i<=10; $i++) {
	my $thisx = int(($i/10)*$w);
	$c->createText($thisx, $h-7,
		       -text => m2km(($i/10)*$dist[$#dist], 1, 1));
    }

    my $thisy = $y_sub->($max_h);
    $c->createText(2, $thisy,
		   -anchor => 'w', -text => int($max_h) . "m");

}

1;

__END__

==> de-BBBike-3.03.tgz: BBBike/BBBikeProfil.pm <==
# -*- perl -*-

#
# $Id: BBBikeProfil.pm,v 1.4 2001/03/25 17:11:20 eserte Exp $
# Author: Slaven Rezic
#
# Copyright (C) 1999 Slaven Rezic. All rights reserved.
# This package is free software; you can redistribute it and/or
# modify it under the same terms as Perl itself.
#
# Mail: eserte@cs.tu-berlin.de
# WWW:  http://user.cs.tu-berlin.de/~eserte/
#

package BBBikeProfil;
use BBBikeUtil;
use strict;
use vars qw();

BEGIN {
    if (!eval '
use blib "/home/e/eserte/src/perl/Msg";
use Msg qw(frommain);
1;
') {
	warn $@ if $@;
	eval 'sub M ($) { $_[0] }';
	eval 'sub Mfmt { sprintf(shift, @_) }';
    }
}

sub new {
    bless {}, $_[0];
}

sub Show {
    my($self, $top, $context, %args) = @_;
    return if $self->{Destroyed};
    my $toplevel;
    if ($context->{ProfilToplevel} &&
	Tk::Exists($context->{ProfilToplevel})) {
	$toplevel = $context->{ProfilToplevel};
    } else {
	$toplevel = $top->Toplevel(-title => M"Profil");
	$toplevel->OnDestroy(sub { $self->{Destroyed} = 1});
	$toplevel->transient($top) if $context->{Transient};
	$context->{ProfilToplevel} = $toplevel;

	my @hooks = qw/new_route del_route/;
	foreach my $hook (@hooks) {
	    Hooks::get_hooks($hook)->add
		    (sub { $self->Show($top, $context, %args) }, "profil");
	}
	$toplevel->OnDestroy
	    (sub {
		 foreach my $hook (@hooks) {
		     Hooks::get_hooks($hook)->del("profil");
		 }
	     }
	    );
    }
    my $c;
    if ($context->{ProfilCanvas} &&
	Tk::Exists($context->{ProfilCanvas})) {
	$c = $context->{ProfilCanvas};
    } else {
	my($w, $h) = (int($top->screenwidth/3*2),
		      int($top->screenheight/6));
	$c = $toplevel->Canvas(-height => $h, -width => $w)->pack;
	$context->{ProfilCanvas} = $c;
    }

    $self->Redraw($context, %args);
}

# XXX bind: bei Klick wird die entsprechende Stelle auf der Karte
# gezeigt. Vielleicht brauche ich hier auch die canvascoords oder
# die transpose-funktion
sub Redraw {
    my($self, $context, %args) = @_;
    my $t = $context->{ProfilToplevel};
    my $c = $context->{ProfilCanvas};
    $c->delete('all');
    my $hoehe   =    $context->{Hoehe};
    my(@coords) = @{ $context->{Coords} };
    my($w, $h) = ($c->cget(-width), $c->cget(-height));
    my(@dist) = (0);
    for(my $i=0; $i<$#coords; $i++) {
	my($x1,$y1, $x2,$y2) = (@{ $coords[$i] },
				@{ $coords[$i+1] });
	my $dist = CORE::sqrt(sqr($x2-$x1)+sqr($x2-$x1));
	push @dist, $dist[$#dist] + $dist;
    }

    my $max_h;
    for(my $i=0; $i<=$#coords; $i++) {
	my($x,$y) = @{ $coords[$i] };
	if (exists $hoehe->{"$x,$y"} &&
	    (!defined $max_h || $max_h < $hoehe->{"$x,$y"})) {
	    $max_h = $hoehe->{"$x,$y"};
	}
    }

    if (!defined $max_h) {
	print STDERR "Es gibt keine Hheninformationen fr diese Route\n";
	return;
    }

    my $x_sub = sub { int($_[0]/$dist[$#dist]*$w) };
    my $y_sub = sub { $h - int($_[0]/($max_h+10)*$h) };

    my($lastx, $lasty);
    for(my $i=0; $i<=$#dist; $i++) {
	my($d) = $dist[$i];
	my($x,$y) = @{ $coords[$i] };
	if (exists $hoehe->{"$x,$y"}) {
	    my($thisx, $thisy);
	    $thisx = $x_sub->($d);
	    $thisy = $y_sub->($hoehe->{"$x,$y"});
	    if (defined $lastx) {
		$c->createLine($lastx, $lasty, $thisx, $thisy);
	    }
	    ($lastx, $lasty) = ($thisx, $thisy);
	}
    }

    for(my $i=0; $i<=10; $i++) {
	my $thisx = int(($i/10)*$w);
	$c->createText($thisx, $h-7,
		       -text => m2km(($i/10)*$dist[$#dist], 1, 1));
    }

    my $thisy = $y_sub->($max_h);
    $c->createText(2, $thisy,
		   -anchor => 'w', -text => int($max_h) . "m");

}

1;

__END__
